//
// Copyright 2023 Wultra s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions
// and limitations under the License.
//

package com.wultra.android.malwarelytics.reactnative

import com.facebook.react.bridge.Arguments
import com.facebook.react.bridge.ReadableMap
import com.wultra.android.antimalware.ApkThreat
import com.wultra.android.antimalware.MalwareFlag
import com.wultra.android.antimalware.smart.SmartProtectionResult
import com.wultra.android.antimalware.update.*
import com.wultra.android.rasp.*

// This file contains functions that converts various objects from native Malwarelytics SDK
// to the JavaScript objects.

fun RootDetection.toJs(): ReadableMap {
    // SystemIntegrityInfo
    return Arguments.createMap()
        .put("isJailbroken", false)
        .put("isRooted", isRooted)
        // RootDetectionInfo
        .put("rootInfo", Arguments.createMap()
            .put("isRootCloaked", isRootCloaked)
            .put("rootDetectionProofs", rootDetectionProofs.map { it.toString() })
            .put("rootDetectionConfidence", rootDetectionConfidence)
            .put("rootCloakDetectionConfidence", rootCloakDetectionConfidence)
        )
}

fun HttpProxyDetection.toJs(): ReadableMap {
    // HttpProxyInfo
    return Arguments.createMap()
        .put("isHttpProxyEnabled", isHttpProxyEnabled)
        .put("isUsingAutoConfig", isUsingAutoConfig())
        .put("host", host)
        .put("port", port)
        .put("exclusionList", exclusionList)
        .put("pacFileUrl", pacFileUrl?.toString())
}

fun ScreenSharingDetection.toJs(): ReadableMap {
    // ScreenSharingInfo
    return Arguments.createMap()
        .put("isScreenShared", isScreenShared)
        .put("numberOfDisplays", numberOfDisplays)
        .put("transientData", transientData.toJs())
        .put("isProblematic", isProblematic)
        .put("isTransientChange", isTransientChange)
}

fun ScreenSharingDetection.TransientScreenSharingData.toJs(): ReadableMap {
    // TransientScreenSharingData
    return Arguments.createMap()
        .put("displayAdded", displayAdded)
        .put("displayRemoved", displayRemoved)
}

fun ScreenReaderDetection.toJs(): ReadableMap {
    // ScreenReaderInfo
    return Arguments.createMap()
        .put("isNotAllowedScreenReaderEnabled", isNotAllowedScreenReaderEnabled)
        .put("notAllowedScreenReaders", notAllowedScreenReaders)
        .put("enabledScreenReaders", enabledScreenReaders)
        .put("installedScreenReaders", installedScreenReaders)
}

fun TapjackingDetection.toJs(): ReadableMap {
    // TapjackingInfo
    return Arguments.createMap()
        .put("isTapjackingBlocked", isTapjackingBlocked)
        .put("tapjackingCapableApps", tapjackingCapableApps)
}

fun EmulatorDetection.toJs(): ReadableMap {
    return Arguments.createMap()
        .put("isEmulator", isEmulator)
        .put("detectedEmulatorType", detectedEmulatorType?.toString())
        .put("emulatorDetectionProofs", emulatorDetectionProofs.map { it.toString() })
}

fun SmartProtectionResult.toJs(): ReadableMap {
    return Arguments.createMap()
        .put("uiDisplayed", uiDisplayed)
        .put("onlineUpdateSucceeded", onlineUpdateSucceeded)
        .put("evaluationSucceeded", evaluationSucceeded)
}

fun ApkThreat.toJs(): ReadableMap {
    return Arguments.createMap()
        .put("packageName", packageName)
        .put("threatIndex", threatIndex.name)
        .put("evaluatedThreatIndex", evaluatedThreatIndex.name)
        .put("suggestedThreatIndex", suggestedThreatIndex.name)
        .put("malwareDetectionName", malwareDetectionName)
        .put("mitigations", mitigations.toJs { it.name })
        .put("reasons", reasons.toJs { it.name })
        .put("threatInstaller", threatInstaller?.name)
        .put("flags", flags.toJs { it.toJs() })
}

fun MalwareFlag.toJs(): ReadableMap {
    return Arguments.createMap()
        .put("name", name)
        .put("type", type.name)
}

fun UpdateInfo.toJs(): ReadableMap {
    val sUpdates = successfulUpdates.toList().fold(Arguments.createMap()) { acc, it -> 
        acc.put(it.first.name, it.second.toJs()) 
    }
    val fUpdates = failedUpdates.toList().fold(Arguments.createMap()) { acc, it -> 
        acc.put(it.first.name, it.second.toJs()) 
    }
    
    // UpdateInfo
    return Arguments.createMap()
        .put("successfulUpdates", sUpdates)
        .put("failedUpdates", fUpdates)
}

fun UpdateInfoSuccess.toJs(): ReadableMap {
    // UpdateInfoSuccess
    return Arguments.createMap()
        .put("lastSuccessTimestamp", lastSuccessTimestamp)
        .put("lastSuccessCheckCount", lastSuccessCheckCount)
        .put("lastSuccessUpdatedCount", lastSuccessUpdatedCount)
}

fun UpdateInfoFailure.toJs(): ReadableMap {
    // UpdateInfoFailure
    return Arguments.createMap()
        .put("lastFailureTimestamp", lastFailureTimestamp)
        .put("lastFailureReason", lastFailureReason)
}

fun ObservedUpdateInfo.toJs(): ReadableMap {
    // ObservedUpdateInfo
    return Arguments.createMap()
        .put("updateResult", updateResult.name)
        .put("updateType", updateType.name)
        .put("checkedApps", checkedApps)
        .put("updatedApps", updatedApps)
        .put("failureReason", failureReason)
}

fun ActiveCallDetection.toJs(): ReadableMap {
    // ActiveCallInfo
    return Arguments.createMap()
        .put("callState", callState.name)
}

fun AppPresenceDetection.toJs(): ReadableMap {
    // AndroidAppPresenceInfo
    val androidAppPresenceInfo = Arguments.createMap()
        .put("remoteDesktopApps", remoteDesktopApps.toJs { it.toJs() })
    // AppPresenceInfo
    return Arguments.createMap()
        .put("androidAppPresenceInfo", androidAppPresenceInfo)
}

fun NamedApkItemInfo.toJs(): ReadableMap {
    return Arguments.createMap()
        .put("displayName", displayName)
        .put("appName", appName)
        .put("packageName", packageName)
        .put("versionName", versionName)
        .put("versionCode", versionCode.toString())
        .put("signatureHash", signatureHash)
}

fun BiometryDetection.toJs(): ReadableMap {
    // BiometryInfo
    return Arguments.createMap()
        .put("biometricStatus", biometricStatus.name)
        .put("androidxLibStatus", androidxLibStatus)
}

fun DebuggerDetection.toJs(): ReadableMap {
    // DebuggerInfo
    return Arguments.createMap()
        .put("isDebuggerAttached", isDebuggerAttached)
        .put("isWaitingForDebugger", isWaitingForDebugger)
        .put("debbuggerType", debbuggerType.toJs { it.name })
}